# The process_azks function takes a folder (and its sub-folders), finds the .AZK files and processes them into
# a list of two elements, both of which are data.frames, one called trials and one called participants

# The parameters are:
# folder - the location of the folder
# adjust_time - function taking a filename src_file and a time dt to adjust for faulty clocks on testing PCs - defaults to returning dt
# trial_specifier - a named list specifying how to interpret the trial code into factors in the data.frame
#                         e.g. splitting 12057 into 1 2 057, and then coding into factors
#                 - format of trial_specifier is that:  
#                     - each item is named for a column desired in the trials data.frame
#                     - each item is itself a list with one or two elements:
#                          - nch, the number of characters of the trial code to use to fill the column, which is compulsory
#                          - labels, a named list of levels of the factors, e.g. list(`1`='word',`2`='nonword')
#                 - these are processed in order left-to-right 
#                         e.g. list(
#                                   lexicality=list(nch=1,labels=list(`1`='word',`2`='nonword')),
#                                   relatedness=list(nch=1,labels=list(`1`='related',`2`='unrelated')),
#                                   item=list(nch=3)
#                                  )                       

process_azks<-function(folder,adjust_time=function(src_file,dt) dt,trial_specifier=list())
{
  # set up columns in trials data.frame
  ret<-data.frame(pp_id_long=character(0),pp_id_short=character(0),date_time=vector(mode="list"),srcfile=character(0),trial_id=character(0),accuracy=logical(0),rt=numeric(0),filler=logical(0))
  ret<-cbind(ret,lapply(trial_specifier,function(x) character(0)))

  # find AZKs
  file_list<-dir(folder,full.names=T,recursive=T,pattern="*.azk")
  
  for(cfile in file_list)
  {
    # filepath without folder passed as argument
    srcfilename<-substr(cfile,nchar(folder)+1,nchar(cfile))
    
    # just the filename
    srcfilebase<-sub(".*/","",cfile)
    
    cat(" --- ")
    cat(srcfilename)
    cat(" --- \n")
    short_id<-""
    long_id<-""
    date_time<-""

    txt<-readLines(cfile)

    for(line in txt)
    {
      lwds<-strsplit(line,split=" ")[[1]]
      if(substr(line,1,8)=="Subject ") # new subject
      {
        cat(line)
        cat("\n")
        lwds<-strsplit(line,split=" ")[[1]]
        lwds<-c(lwds,"","")
        long_id<-paste(sep="_",srcfilename,paste(collapse="_",lwds[c(3,4,14)]))
        short_id<-lwds[14]
        date_time<-adjust_time(cfile,strptime(format="%m/%d/%Y %H:%M:%S",paste(collapse=" ",lwds[3:4])))
        
      } else if(nchar(line)==0||substr(line,1,1)=="*"||substr(line,1,1)=="!"||substr(line,1,8)=="Subjects"||substr(line,1,6)=="  Item"||substr(line,1,4)=="Data")
      {
        next; # error or comment
      }
      else
      {
        trial_id<-sub(" *$","",substr(line,1,8)) # trial IDs are left padded
        trial_specs<-data.frame()
        if(length(trial_specifier)>0)
        {
          startstop<-lapply(trial_specifier,`[[`, 'nch')
          startstop<-cumsum(startstop) # calculate relevant column numbers
          trial_specs<-substring(trial_id,1+c(0,startstop[-length(startstop)]),startstop)
          trial_specs<-as.list(trial_specs)
          names(trial_specs)<-names(trial_specifier) 
        }
        
        art<-as.numeric(gsub(" ","",substr(line,8,100))) # RTs are numbers
        acc<-art>0 # negative numbers mean error
        rt<-abs(art) # negative numbers are postive RTs (on error trials)
        thistrial<-data.frame(pp_id_long=long_id,pp_id_short=short_id,date_time=date_time,srcfile=srcfilebase,trial_id=trial_id,accuracy=acc,rt=rt,filler=F)
        thistrial<-cbind(thistrial,trial_specs)
        ret<-rbind(ret,thistrial)
      }
    }
  }
  if(length(trial_specifier)>0)
  {
    for(i in names(trial_specifier))
    {
      if(!is.null(trial_specifier[[i]]$labels)) # apply labels if they exist
      {
        ret[[i]]<-factor(ret[[i]],levels=names(trial_specifier[[i]]$labels),labels=trial_specifier[[i]]$labels)
        ret$filler<-ret$filler|is.na(ret[[i]])
      }
    }
  }
  
  pp<-with(subset(ret,!filler),aggregate(accuracy,list(long_id=pp_id_long,short_id=pp_id_short,date_time=date_time,srcfile=srcfile),mean))
  names(pp)[5]<-"mean_accuracy"
  
  list(participants=pp,trials=ret)
}

fix_time<-function(fn,dt) # this adjusts for the Cognitive Science of Reading Lab's PCs' clock problems
{
#  print(fn)
  if(grepl("Computer1",fn)) return(dt+60*60*24*339+60*60*9.75)
  if(grepl("Computer2",fn)) return(dt+60*60*24*1734+60*60*8.4)
  if(grepl("Computer3",fn)) return(dt+60*60*24*354+60*60*19)
}

# Process the present data, coded the same way as Expts 1 & 2
process_azks(".",fix_time,
             list(lexicality=list(nch=1,labels=list(`1`="word",`2`="nonword")),
                  grouping=list(nch=1),
                  condition=list(nch=1,labels=list(`5`="identity",`6`="neighbourly",`7`="control")),
                  group=list(nch=3)))->thedata

# As with Expts 1 & 2, the final 3 digits are the PAIR of targets, and the second digit differentiates the two members of that pair
thedata$trials$item<-as.factor(paste(sep="",thedata$trials$grouping,thedata$trials$group))

# Given we had no intention of analysing the non-native data, some RAs did not save it, but others did
# Also some RAs forgot to record participant numbers
# These participants were identified from the paper record of ID & first language
# in combination with the schedule of participants where the ID is missing (hence the need to correct the clocks)
nonnatives<-c(
  "",                                                         # 2 not saved
  "/Computer2/neighborly6/CB_6_3.azk_03/05/2017_04:46:22_3",  # 3
  "/Computer2/neighborly6/CB_6_4.azk_03/05/2017_05:55:03_4",  # 4
  "/Computer1/neighborly6/CB_6_3.azk_01/03/2021_07:20:34_",   #15
  "",                                                         #25 not saved
  "/Computer2/neighborly6/CB_6_2.azk_03/12/2017_04:52:43_26", #26
  "/Computer3/neighborly6/CB_6_4.azk_01/23/2021_20:33:03_31", #31
  "/Computer3/neighborly6/CB_6_3.azk_01/23/2021_21:45:37_34", #34
  "/Computer2/neighborly6/CB_6_4.azk_04/15/2017_09:30:52_35", #35
  "/Computer2/neighborly6/CB_6_2.azk_04/20/2017_09:23:22_40", #40
  "/Computer1/neighborly6/CB_6_3.azk_02/08/2021_23:18:56_42", #42
  "/Computer1/neighborly6/CB_6_1.azk_02/09/2021_04:07:58_",   #46
  "/Computer2/neighborly6/CB_6_2.azk_04/19/2017_07:53:59_49", #49
  "/Computer1/neighborly6/CB_6_1.azk_02/15/2021_23:25:28_67", #67
  "/Computer1/neighborly6/CB_6_2.azk_02/16/2021_04:29:55_",   #69
  "/Computer1/neighborly6/CB_6_3.azk_02/16/2021_05:27:06_",   #71
  "/Computer1/neighborly6/CB_6_4.azk_02/16/2021_06:42:53_",   #73
  "/Computer2/neighborly6/CB_6_2.azk_04/26/2017_07:47:04_76", #76
  "/Computer3/neighborly6/CB_6_3.azk_02/05/2021_17:15:28_",   #82
  "/Computer3/neighborly6/CB_6_4.azk_02/05/2021_17:43:02_",   #83
  "/Computer2/neighborly6/CB_6_1.azk_04/28/2017_06:01:52_",   #84
  "/Computer1/neighborly6/CB_6_4.azk_02/23/2021_05:16:27_",   #88
  "/Computer2/neighborly6/CB_6_2.azk_05/03/2017_06:15:17_90", #90
  "/Computer1/neighborly6/CB_6_2.azk_02/27/2021_06:14:29_97", #97
  "/Computer3/neighborly6/CB_6_2.azk_02/11/2021_21:14:17_98", #98
  "/Computer2/neighborly6/CB_6_2.azk_05/04/2017_09:30:25_99", #99
  "/Computer1/neighborly6/CB_6_2.azk_02/28/2021_03:20:24_",   #104
  "/Computer2/neighborly6/CB_6_3.azk_05/05/2017_06:25:51_",   #105
  "/Computer1/neighborly6/CB_6_1.azk_03/05/2021_05:20:53_112",#112
  "/Computer3/neighborly6/CB_6_1.azk_02/18/2021_20:03:01_115",#115
  "", "", "",                                                 #120, 121, 122 not saved
  "/Computer1/neighborly6/CB_6_4.azk_03/09/2021_05:01:15_125",#125

    # Participant ID sequence was run separately on Jan 12, 2022 due to no access to the original list of participants
  "/Computer2/neighborly6/CB_6_2.azk_04/14/2017_04:28:03_",   #2 from Jan 12
  "/Computer3/neighborly6/CB_6_4.azk_01/22/2021_17:42:53_"   #4 from Jan 12
)


# This participant was supposed to take part in an unrelated (repeated-letters) LDT
# experiment on 16th Feb 2022 and had already participated in this one on 31st Jan
other_errors<-"/Computer2/neighborly6/CB_6_2.azk_05/19/2017_06:15:32_"

# #135 is missing for the converse error, but was a non-native speaker anyway

# apply native language and accuracy criteria
valid_pps<-subset(thedata$participants,!long_id%in%nonnatives&!long_id%in%other_errors&mean_accuracy>=.75)$long_id

# check balance
table(subset(thedata$participants,long_id%in%valid_pps)$srcfile)

drop_for_balance<-c(
  "/Computer3/neighborly6/CB_6_3.azk_05/15/2021_21:29:34_136", #136
  "/Computer2/neighborly6/CB_6_2.azk_05/18/2017_08:08:20_128", #128
  "/Computer3/neighborly6/CB_6_2.azk_02/17/2021_22:16:19_114" #114
)
# balance the counterbalancing lists
valid_pps<-subset(thedata$participants,!long_id%in%nonnatives&!long_id%in%other_errors&!long_id%in%drop_for_balance&mean_accuracy>=.75)$long_id
table(subset(thedata$participants,long_id%in%valid_pps)$srcfile)

# apply the item accuracy cut-off
item_accs<-with(subset(thedata$trials,pp_id_long%in%valid_pps&!filler&lexicality=="word"),aggregate(accuracy,list(item=item),mean))
valid_items<-item_accs$item[item_accs$x>=.6]

clean_trials_exp3<-subset(thedata$trials,pp_id_long%in%valid_pps&!filler&lexicality=="word"&rt>150&rt<1500&item%in%valid_items)

# mean RTs
with(clean_trials_exp3,aggregate(rt,list(condition=condition),mean))

require(lme4)
require(optimx)
require(car)
require(phia)

# full model - singular
m3F<-lmer(rt~condition+(condition|pp_id_long)+(condition|item),
         clean_trials_exp3,
         control=lmerControl(optimizer="bobyqa",optCtrl=list()))

# other singular models
m3NC<-lmer(rt~condition+(condition||pp_id_long)+(condition||item),
          clean_trials_exp3,
          control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=1e6)))

m3I<-lmer(rt~condition+(condition|pp_id_long)+(1|item),
         clean_trials_exp3,
         control=lmerControl(optimizer="bobyqa",optCtrl=list()))

m3S<-lmer(rt~condition+(1|pp_id_long)+(condition|item),
         clean_trials_exp3,
         control=lmerControl(optimizer="bobyqa",optCtrl=list()))

# converged model
m3<-lmer(rt~condition+(1|pp_id_long)+(1|item),
         clean_trials_exp3,
         control=lmerControl(optimizer="bobyqa",optCtrl=list()))

# exploration of source of singular terms
m3NCalt<-lmer(rt~condition+(1|pp_id_long)+(1|item)
              +(0+I(1*(condition=="identity"))|pp_id_long)+(0+I(1*(condition=="identity"))|item)
              +(0+I(1*(condition=="neighbourly"))|pp_id_long)+(0+I(1*(condition=="neighbourly"))|item),
              clean_trials_exp3,
              control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))

m3NCalt_2<-lmer(rt~condition+(1|pp_id_long)+(1|item)
              +(0+I(1*(condition=="identity"))|pp_id_long)+(0+I(1*(condition=="identity"))|item)
              +(0+I(1*(condition=="neighbourly"))|item),
              clean_trials_exp3,
              control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))

m3NCalt_3<-lmer(rt~condition+(1|pp_id_long)+(1|item)
              +(0+I(1*(condition=="identity"))|pp_id_long)+(0+I(1*(condition=="identity"))|item),
              clean_trials_exp3,
              control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))

m3alt_3<-lmer(rt~condition
                +(I(condition=="identity")|pp_id_long)+(I(condition=="identity")|item),
                clean_trials_exp3,
                control=lmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))


# conclusion: no variability in contrast between neighbourly and control stimuli; neither over subjects nor items

anova(m3F,m3) # evidence for random slopes
anova(m3F,m3alt_3) # but only for those contrasting identity and control stimuli

# reported slope-less model - Omnibus ANOVA
Anova(m3)
# only the neighbourly-control contrast is of interest, but adjustment="none" is more convenient than constructing a custom contrast.
testInteractions(m3,adjustment="none")

# more complex models lead to same conclusion
Anova(m3F)
testInteractions(m3F,adjustment="none")
Anova(m3alt_3) # reported in footnote
testInteractions(m3alt_3,adjustment="none") # reported in footnote
